@@ -40,14 +40,28 @@ function [s12, dir1, dir2, m12, M12, M21, count] = distance(t, pos1, pos2)
 
 % Copyright (c) Charles Karney (2024) <karney@alum.mit.edu>.
 
+% Biggest error in distance with test set is for
+%  -89 -179 125.264389682754654315
+%  -90 0 0.00508935298572015
+% 2.41969095257529344553 1.71451268367223232925 0.0001845474104102257 0.00006279539595941223
+% Computed distance = 2.419690952576201, error = 9.1e-13 (MATLAB), 7.1e-13 (Octave)
+
+% Next biggest
+%   90 0 90.015261865118026289
+%   87 179 154.751620586596881957
+% 2.42055166682738704032 1.71451199706494557219 -0.00043933375632808661 -0.00129295666346354547
+% Computed distance = 2.420551666827091, error = 3.0e-13 (MATLAB), 6.7e-13 (Octave)
+
+% Next biggest error is = 2.0e-13
+% Mean error in distance is 1e-14
   Z = 0 * (pos1 + pos2);
   n = size(Z, 1);
   pos1 = pos1 + Z;
   pos2 = pos2 + Z;
   ellip = size(Z, 2) == 2;
   if ellip
-    r1 = elliptocart2(t, pos1);
-    r2 = elliptocart2(t, pos2);
+    r1 = elliptocart2(t, triaxial.ellipnorm(pos1));
+    r2 = elliptocart2(t, triaxial.ellipnorm(pos2));
   else
     [r1, r2] = deal(pos1, pos2);
   end
@@ -60,8 +74,8 @@ function [s12, dir1, dir2, m12, M12, M21, count] = distance(t, pos1, pos2)
   end
   s12 = s12 * t.b; m12 = m12 * t.b;
   if ellip
-    [~, dir1] = cart2toellip(t, r1, v1);
-    [~, dir2] = cart2toellip(t, r2, v2);
+    [~, dir1] = cart2toellip(t, pos1, v1);
+    [~, dir2] = cart2toellip(t, pos2, v2);
   else
     [dir1, dir2] = deal(v1, v2);
   end
@@ -76,7 +90,10 @@ function [s12, v1, v2, m12, M12, M21, count] = distanceint(t, r1, r2)
   if ~(s12 > sqrt(eps))
     if s12 == 0
       ellip = cart2toellip(t, (r1+r2)/2);
-      [~, v1] = elliptocart2(t, ellip, 0);
+      % head north/south in southern/northern hemisphere
+      % east on the equator
+      [~, v1] = elliptocart2(t, ellip, ...
+                             180 * (ellip(1) > 0) + 90 * (ellip(1) == 0));
     end
     [~, v2] = cart2norm(t, r2, v1);
     [~, v1] = cart2norm(t, r1, v1);
@@ -147,7 +164,7 @@ function [s12, v1, v2, m12, M12, M21, count] = distanceint(t, r1, r2)
   % Reflect about Z=0 if Z1 > 0.
   flip = [rr(1,1) < 0 | (rr(1,1) == 0 & rr(2,1) < 0), ...
           rr(1,2) < 0 | (rr(1,2) == 0 & rr(2,2) < 0), ...
-          rr(1,3) > 0];                   % Logical version
+          rr(1,3) > 0];                 % Logical version
   flip = 1 - 2*flip;                    % +/-1 version
   rr = rr .* flip + 0;                  % convert -0 to +0
   M = M * diag(flip);
@@ -176,17 +193,17 @@ function [s12, v1, v2, m12, M12, M21, count] = distanceint(t, r1, r2)
   done = false;                         % A marker
 
   if bet1 == -90 && omg1 == 0
-    % point 1 is an umbilic point at (bet,omg) = (-90,0)
+    % point 1 is an umbilical point at (bet,omg) = (-90,0)
     assert(r2(2) >= 0);
     if vecabs(r1 + r2) == 0
       % point 2 is opposite umbilic (bet,omg) = (90,180)
       % this handles prolate and oblate cases too
       r0 = [0, 1, 0];
-      v0 = [-sqrt(t.kp2), 0, sqrt(t.k2)];   % Already normalized
+      v0 = [-sqrt(t.kp2), 0, sqrt(t.k2)]; % Already normalized
       cond = [2, 0, -1];                % Check crossing Y = 0 plane
       [~, v2, sa] = hybridint(t, r0, v0, cond);
       s12 = 2*sa;
-      % m12 = 0 for any geodesics between opposite umbilic points
+      % m12 = 0 for any geodesics between opposite umbilical points
       m12 = 0;
       % M12 = M21 = -1 only for this symmetric case going through [0, 1, 0].
       M12 = -1; M21 = -1;
@@ -205,7 +222,7 @@ function [s12, v1, v2, m12, M12, M21, count] = distanceint(t, r1, r2)
   end
   if r1(2) == 0 && r2(2) == 0 && ~done
     % Both points on middle ellipse.  This excludes the case of two
-    % opposite umbilic points (already treated above).
+    % opposite umbilical points (already treated above).
     E = signx(r1(3) * r2(1) - r1(1) * r2(3)); % Better to call this an SE flag.
     [~, v1] = cart2norm(t, r1, -[1, 0, 1] * E);
     cond = [14, 0, -1];
@@ -245,7 +262,7 @@ function [s12, v1, v2, m12, M12, M21, count] = distanceint2(t, r1, r2)
 % For prolate: bet2 = 0, omg1 <= abs(omg2) <= 180-omg1
 %
 % and the points aren't very close
-% and neither point is an umbilic point (or pole)
+% and neither point is an umbilical point (or pole)
 
 % If r1(2) = 0, then either bet1 = -90 or omg1 = 0.
 
@@ -256,10 +273,10 @@ function [s12, v1, v2, m12, M12, M21, count] = distanceint2(t, r1, r2)
 
 % Now r2(2) > 0.
 
-% If bet1 = -90, then there are two umbilic geodesics U[12] and U[34].
+% If bet1 = -90, then there are two umbilical geodesics U[12] and U[34].
 % with omgu[34] = 0, omgu[12] = 180.  alp1 lies in in (-90, 90)
 
-% If omg1 = 0, then there are two umbilic geodesics U[14] and U[23] and
+% If omg1 = 0, then there are two umbilical geodesics U[14] and U[23] and
 % omgu[14] = 0, check omgu[23] = 180.  alp1 lies in (0, 180).
 
 % If r1 doesn't lie on Y = 0 then there are distinct four umbilical
@@ -285,7 +302,7 @@ function [s12, v1, v2, m12, M12, M21, count] = distanceint2(t, r1, r2)
 
 % For bet2 = bet1, omgu1 = omgu4 = omg1.  Replace alp1u = 90, alp4u = -90.
 % For omg2 > 0, we have E (because omg2 > omg1) and alp1 is in (alp1u,
-% alp2u) (no need to compute umbilic geodesics).  For omg2 < 0, compute
+% alp2u) (no need to compute umbilical geodesics).  For omg2 < 0, compute
 % omgu2 first if omg2 < -90.
 
   fzerop = false;
@@ -331,6 +348,7 @@ function [s12, v1, v2, m12, M12, M21, count] = distanceint2(t, r1, r2)
 
   range = [];
   cond = [];
+  betp = 0;
   omgp = 0;
   if t.kp2 == 0
     % Oblate.
@@ -357,6 +375,7 @@ function [s12, v1, v2, m12, M12, M21, count] = distanceint2(t, r1, r2)
     if bet2 == 90
       cond = [2, 0, -1];                  % crossing Y = 0 plane
     else
+      betp = 1;
       cond = [11, bet2, 1];
     end
   elseif bet2 == bet1 && omg2 > 0
@@ -375,9 +394,9 @@ function [s12, v1, v2, m12, M12, M21, count] = distanceint2(t, r1, r2)
     omgp = 1;
   else
     % That's the end of the cases where we can avoid computing umbilics
-    % Last test in condu is to stop geodesic at umbilic point (needed if
+    % Last test in condu is to stop geodesic at umbilical point (needed if
     % bet2 is close to 90).
-    condu = [11, bet2, 1; 14, 0, -1];
+    condu = [11, bet2, 1];
     if bet2 == bet1
       omgu(1) = omg1; omgu(4) = omg1;
       alpu(1) = 90; alpu(4) = -90;
@@ -403,11 +422,12 @@ function [s12, v1, v2, m12, M12, M21, count] = distanceint2(t, r1, r2)
         if isnan(omgu(q))
           alp1 = alpu(q);
           [~, v1] = elliptocart2(t, [bet1, omg1], alp1);
-          r2t = hybridint(t, r1, v1, condu, 0, umb(q,:));
+          betpu = (q == 4 | q == 1) + 2 * (q == 2 | q == 3);
+          r2t = hybridint(t, r1, v1, condu, 2*betpu);
           ellip2t = cart2toellip(t, r2t);
           [bet2t, omgu(q)] = deal(ellip2t(1), ellip2t(2)); %#ok<ASGLU>
           if check
-            assert(abs(bet2t - bet2) < 512 * slopa);
+            assert(abs(bet2t - bet2) < 1024 * slopa);
           end
           done = abs(omg2 - omgu(q)) <= slopa;
           if done, break; end
@@ -419,7 +439,7 @@ function [s12, v1, v2, m12, M12, M21, count] = distanceint2(t, r1, r2)
         omgs(omgs < 0) = omgs(omgs < 0) + 360;
       end
       % Replace < by <= in case the addition of 360 causes equality.  This is
-      % probably caught by the fussy test on omg2 == omgu(q) above.
+      % probably caught by the fuzzy test on omg2 == omgu(q) above.
       if omgs(1) <= omgs(2) && omgs(2) <= omgs(3)
         % Good ranking omgs(1) < omg2(2) < omgs(3)
         switch k
@@ -432,6 +452,7 @@ function [s12, v1, v2, m12, M12, M21, count] = distanceint2(t, r1, r2)
         end
         range = [alpu(k), alpu(l)];
         omgp = k == 1;
+        betp = (k == 4) + 2 * (k == 2);
         break;
       end
     end
@@ -442,7 +463,7 @@ function [s12, v1, v2, m12, M12, M21, count] = distanceint2(t, r1, r2)
     if omgp && omg2 < 0, omg2 = omg2 + 360; end
 
     cond = [cond; 7, 0, -1];
-    fun = @(alp1t) domgf(alp1t, t, cond, r1, bet1, omg1, omg2, omgp);
+    fun = @(alp1t) domgf(alp1t, t, cond, r1, bet1, omg1, omg2, omgp+2*betp);
     if fzerop
       % Maybe narrow range slightly, since fzero evaluates fun at the
       % end points.
@@ -457,16 +478,16 @@ function [s12, v1, v2, m12, M12, M21, count] = distanceint2(t, r1, r2)
   [~, v2, s12, m12, M12, M21] = hybridint(t, r1, v1, cond, 0, r2);
 end
 
-function [domg, domgp] = domgf(alp1t, t, cond, r1, bet1, omg1, omg2, omgp)
-  if nargin < 8, omgp = 0; end
+function [domg, domgp] = domgf(alp1t, t, cond, r1, bet1, omg1, omg2, altp)
+  if nargin < 8, altp = 0; end
   [~, v1] = elliptocart2(t, [bet1, omg1], alp1t);
-  [r2, v2, ~, m12] = hybridint(t, r1, v1, cond, omgp);
+  [r2, v2, ~, m12] = hybridint(t, r1, v1, cond, altp);
   if size(cond, 1) > 0 && cond(1,1) == 2
     % If Y = 0 is the stopping condition, make this so
     r2(2) = cond(1,2);
   end
   [ellip2t, alp2, ~, rad2] = cart2toellip(t, r2, v2); omg2t = ellip2t(2);
-  if omgp && omg2t < 0, omg2t = omg2t + 360; end
+  if bitand(altp, 1) && omg2t < 0, omg2t = omg2t + 360; end
   domg = omg2t - omg2;
   domgp = m12 / ( cosd(alp2) * rad2 );
 end
